@@ -117,6 +117,7 @@ void usage()
    -f               stay in the foreground, don't run as a daemon\n\
    -g x             run as group <x>\n\
    -p x             use pid file <x> instead of the default\n\
+                    '-' means not tu use any pid file\n\
    -u x             run as user <x>\n"
 ));
 }
@@ -196,6 +197,7 @@ int main(int argc, char *argv[])
         if (user == NULL) { user = USER; }
         if (group == NULL) { group = GROUP; }
         if (pid_file == NULL) { pid_file = PIDFILE; }
+        if (!strcmp(pid_file, "-")) { pid_file = NULL; }
 
         if ((passwd_struct = getpwnam(user)) == NULL)
         {
@@ -209,16 +211,16 @@ int main(int argc, char *argv[])
             exit(1);
         }
         gid = group_struct->gr_gid;
-        if (setuid(uid) != 0)
-        {
-            DEBUG(_("Error setting uid to %u: %s"), uid, strerror(errno));
-            exit(1);
-        }
         if (setgid(gid) != 0)
         {
             DEBUG(_("Error setting gid to %u: %s"), gid, strerror(errno));
             exit(1);
         }
+        if (setuid(uid) != 0)
+        {
+            DEBUG(_("Error setting uid to %u: %s"), uid, strerror(errno));
+            exit(1);
+        }
     }
 
     /* parse conf file */
@@ -245,26 +247,33 @@ int main(int argc, char *argv[])
             DEBUG(_("Daemonize error: %s"), strerror(errno));
             exit (1);
         }
+
+        halevt_fork = 1;
+
         if (pid_file == NULL)
         {
-            DEBUG(_("No pid file specified, no pid file used"));
+            DEBUG(_("No pid file used"));
         }
         else
         {
             FILE *file = fopen (pid_file, "w");
+            if (file == NULL)
+            {
+                DEBUG(_("Open pid file %s failed: %s"), pid_file, strerror(errno));
+                exit (1);
+            } 
             if (lockf(fileno(file), F_TLOCK, 0) != 0)
             {
                 DEBUG(_("Lock %s failed: %s"), pid_file, strerror(errno));
                 exit (1);
             }
-            if ((fprintf (file, "%d", getpid())) < 0)
+            if ((fprintf (file, "%d\n", getpid())) < 0)
             {
                 DEBUG(_("Writing pid to %s failed"), pid_file);
                 exit (1);
             }
             fclose(file);
         }
-        halevt_fork = 1;
     }
 
     halevt_run_oninit ();
